@@ -2,7 +2,7 @@
 """
 image2gcode: convert an image to gcode.
 """
-__version__ = "2.4.0"
+__version__ = "2.6.2"
 
 import os
 import sys
@@ -32,14 +32,19 @@ def loadImage(args):
         img_background = Image.new(mode = "RGBA", size = img.size, color = (255,255,255))
         img = Image.alpha_composite(img_background, img)
 
+        if args.size is not None:
+            # convert image to black&white (without alpha) and requested size
+            img = img.resize((int(args.size[0]/args.pixelsize),
+                              int(args.size[1]/args.pixelsize)), Image.Resampling.LANCZOS).convert("L")
+        else:
+            # convert image to black&white
+            # Note that the image keeps its resolution (same number of pixels for x and y-axis)
+            #  each image pixel is converted to a gcode move of pixelsize length
+            img = img.resize(img.size, Image.Resampling.LANCZOS).convert("L")
+
         if args.showimage:
             img.show()
 
-        # convert image to black&white
-        # Note that the image keeps its resolution (same number of pixels for x and y-axis)
-        #  each image pixel is converted to a gcode move of pixelsize length
-        img = img.resize(img.size, Image.Resampling.LANCZOS).convert("L")
-
         # return np array of image
         return np.array(img)
 
@@ -88,8 +93,15 @@ def image2gcode(img, args) -> str:
     # header comment
     gcode = [f";    {os.path.basename(sys.argv[0])} {__version__} ({str(datetime.now()).split('.')[0]})\n"
              f';    Area: {str(round(img.shape[1] * args.pixelsize,2))}mm X {str(round(img.shape[0] * args.pixelsize,2))}mm (XY)\n'
-             f';    > pixelsize {str(args.pixelsize)}mm^2, speed {str(args.speed)}mm/min, maxpower {str(args.maxpower)},\n'
-             f";      speedmoves {args.speedmoves}, noise level {args.noise}, offset {args.offset}, burn mode {'M3' if args.constantburn else 'M4'}\n;\n"]
+             f';    arguments:\n'
+             f';      pixelsize: {str(args.pixelsize)}mm^2,\n'
+             f';      speed: {str(args.speed)}mm/min,\n'
+             f';      maxpower: {str(args.maxpower)},\n'
+             f";      speedmoves: {args.speedmoves},\n"
+             f";      noise level: {args.noise},\n"
+             f";      offset: {args.offset},\n"
+             f";      burn mode: {'M3' if args.constantburn else 'M4'},\n"
+             f';      overscan {args.overscan}\n']
 
     # init gcode
     gcode += ["G00 G17 G40 G21 G54","G90"]
@@ -155,12 +167,12 @@ def image2gcode(img, args) -> str:
             if count == 0:
                 # delay emit first pixel (so all same power pixels can be emitted in one sweep)
                 prev_pow = laserpow
-                # set last head loaction on start of the line
+                # set last location on start of the line
                 lastloc = (X,Y)
 
             # draw pixels until change of power
             if laserpow != prev_pow or count == line.size-1:
-                #if prev_pow != 0:
+                # skip all zero power points
                 if prev_pow > args.noise:
                     code = ""
                     if lastloc:
@@ -171,7 +183,18 @@ def image2gcode(img, args) -> str:
                             code += f"G1\n"
                         else:
                             # normal speed
-                            code = f"X{lastloc[0]}Y{lastloc[1]} S0\n"
+                            if args.overscan and head[1] != Y:
+                                if left2right:
+                                    # go to next line and start overscan pixels earlier
+                                    min_X = round(args.offset[0], XY_prec)
+                                    overscanpixels = round(lastloc[0] - (args.overscan * args.pixelsize), XY_prec)
+                                    overscanpixels = overscanpixels if overscanpixels > min_X else min_X
+                                else:
+                                    max_X = round(args.offset[0] + (line.size * args.pixelsize), XY_prec)
+                                    overscanpixels = round(lastloc[0] + (args.overscan * args.pixelsize), XY_prec)
+                                    overscanpixels = overscanpixels if overscanpixels < max_X else max_X
+                                code = f"X{overscanpixels}Y{lastloc[1]}S0\n"
+                            code += f"X{lastloc[0]}Y{lastloc[1]}S0\n"
 
                     # emit point
                     code += f"X{X}S{prev_pow}"
@@ -188,6 +211,22 @@ def image2gcode(img, args) -> str:
                     lastloc = (X,Y)
 
                 if count == line.size-1:
+                    # overscan this line (if we can)
+                    if args.overscan and head[1] == Y and lastloc is not None:
+                        # one or more points are drawn on this line, but one or more zero power pixels are not drawn
+                        if left2right:
+                            overscanpixels = min(round(head[0] + (args.overscan * args.pixelsize), XY_prec),
+                                                 round(args.offset[0] + (line.size * args.pixelsize), XY_prec))
+                        else:
+                            overscanpixels = max(round(head[0] - (args.overscan * args.pixelsize), XY_prec),
+                                                 round(args.offset[0], XY_prec))
+
+                        # emit overscan pixels
+                        code = f"X{overscanpixels}S0"
+                        gcode += [code]
+
+                        # head at this location
+                        head = (overscanpixels,Y)
                     continue
                 prev_pow = laserpow # if laserpow != prev_pow
 
@@ -218,6 +257,7 @@ def main() -> int:
     power_default = 300
     speedmoves_default = 10
     noise_default = 0
+    overscan_default = 0
 
     # Define command line argument interface
     parser = argparse.ArgumentParser(description='Convert an image to gcode for GRBL v1.1 compatible diode laser engravers,\n'
@@ -232,18 +272,24 @@ def main() -> int:
         type=int, help='draw speed in mm/min')
     parser.add_argument('--maxpower', default=power_default, metavar="<default:" +str(power_default)+ ">",
         type=int, help="maximum laser power while drawing (as a rule of thumb set to 1/3 of the maximum of a machine having a 5W laser)")
-    parser.add_argument('--offset', default=[10.0, 10.0], nargs=2, metavar=('X-off', 'Y-off'),
-        type=float, help="laser drawing starts at offset (default: X10.0 Y10.0)")
+    parser.add_argument('--size', default=None, nargs=2, metavar=('gcode-width', 'gcode-height'),
+        type=float, help="target gcode width and height in mm (default: not set and determined by pixelsize and image source resolution)")
+    parser.add_argument('--offset', default=None, nargs=2, metavar=('X-off', 'Y-off'),
+        type=float, help="laser drawing starts at offset in mm (default not set, --center cannot be set at the same time)")
+    parser.add_argument('--center', action='store_true', default=False, help='set origin at the image center (--offset cannot be set at the same time)' )
     parser.add_argument('--speedmoves', default=speedmoves_default, metavar="<default:" + str(speedmoves_default)+">",
         type=float, help="length of zero burn zones in mm (0 sets no speedmoves): issue speed (G0) moves when skipping space of given length (or more)")
     parser.add_argument('--noise', default=noise_default, metavar="<default:" +str(noise_default)+ ">",
         type=int, help="noise power level, do not burn pixels below this power level")
+    parser.add_argument('--overscan', default=overscan_default, metavar="<default:" +str(overscan_default)+ ">",
+        type=int, help="overscan image lines to avoid incorrect power levels for pixels at left and right borders, number in pixels, default off")
     parser.add_argument('--constantburn', action='store_true', default=False, help='select constant burn mode M3 (a bit more dangerous!), instead of dynamic burn mode M4')
     parser.add_argument('--validate', action='store_true', default=False, help='validate gcode file, do inverse and show image result' )
     parser.add_argument('-V', '--version', action='version', version='%(prog)s ' + __version__, help="show version number and exit")
 
     args = parser.parse_args()
 
+    # check incompatible arguments
     if args.validate and (args.gcode.name == "<stdout>" or not GCODE2IMAGE):
         if not GCODE2IMAGE:
             print("For validation package gcode2image.py must be installed (in $PATH or python install path)")
@@ -252,13 +298,24 @@ def main() -> int:
             parser.print_usage(sys.stderr)
         sys.exit(1)
 
+    if args.center and args.offset is not None:
+        print("options --center and --offset cannot be combined!")
+        sys.exit(1)
+
+    if args.offset is None:
+        args.offset = (0.0, 0.0)
+
     # load and convert image to B&W
     # flip image updown because Gcode and raster image coordinate system differ
     narr = np.flipud(loadImage(args))
 
+    if args.center:
+        args.offset = (-round((narr.shape[1] * args.pixelsize)/2,1), -round((narr.shape[0] * args.pixelsize)/2,1))
+
     print(f'Area: {str(round(narr.shape[1] * args.pixelsize,2))}mm X {str(round(narr.shape[0] * args.pixelsize,2))}mm (XY)\n'
           f'    > pixelsize {args.pixelsize}mm^2, speed {args.speed}mm/min, maxpower {str(args.maxpower)},\n'
-          f"      speedmoves {args.speedmoves}, noise level {args.noise}, offset {args.offset}, burn mode {'M3' if args.constantburn else 'M4'}")
+          f"      speedmoves {args.speedmoves}, noise level {args.noise}, offset {args.offset}, burn mode {'M3' if args.constantburn else 'M4'},"
+          f' overscan {args.overscan}')
 
     # emit gcode for image
     print(image2gcode(narr, args), file=args.gcode)
@@ -274,8 +331,6 @@ def main() -> int:
 
             # show image
             img.show()
-            # write image file (png)
-            #pic.save(args.gcode.name.split('.')[0] + '_' + sys.argv[0].split('.')[0] + '.png')
 
     return 0
 
